from unittest import TestCase
from terraform_compliance.main import ReadableDir, cli
from tests.mocks import MockedArgumentParser, MockedArgHandling
from mock import patch
import os


class TestMain(TestCase):

    def test_readable_dir_call_git_repository_success(self):
        class Namespace(object):
            pass

        resp = ReadableDir('parser', 'value', 'git:value').__call__('parser', Namespace, 'git:value')
        self.assertEqual(Namespace.value, 'value')
        self.assertTrue(resp)


    @patch.object(os.path, 'isdir', return_value=False)
    def test_readable_dir_call_not_a_local_directory(self, *args):
        class Namespace(object):
            pass

        with self.assertRaises(SystemExit):
            ReadableDir('parser', 'value', 'non_existing_dir').__call__('parser', Namespace, 'non_existing_dir')

    @patch.object(os.path, 'isdir', return_value=True)
    @patch.object(os, 'access', return_value=True)
    def test_readable_dir_call_is_a_local_dir(self, *args):
        class Namespace(object):
            pass

        resp = ReadableDir('parser', 'value', 'value').__call__('parser', Namespace, 'value')
        self.assertEqual(Namespace.value, 'value')
        self.assertTrue(resp)

    @patch.object(os.path, 'isdir', return_value=True)
    @patch.object(os, 'access', return_value=False)
    def test_readable_dir_call_can_not_access_dir(self, *args):
        class Namespace(object):
            pass

        with self.assertRaises(SystemExit):
            ReadableDir('parser', 'value', 'non_accessible_dir').__call__('parser', Namespace, 'non_accessible_dir')

    # Unfortunately current cli() structure doesn't allow us to test everything that we want.
    # These tests needs to be refactored when cli() is suitable for tests.

    @patch('terraform_compliance.main.mkdtemp', return_value='/tmp/something')
    @patch('terraform_compliance.main.copy_tree', return_value=True)
    @patch('terraform_compliance.main.rmtree', return_value=True)
    @patch('terraform_compliance.main.Repo', return_value=True)
    @patch('terraform_compliance.main.load_tf_files', return_value=True)
    @patch('terraform_compliance.main.call_radish', return_value=True)
    def test_cli_use_local_dirs_for_all(self, *args):
        self.assertTrue(cli(MockedArgHandling(), MockedArgumentParser('test', 'test description')))

    @patch('terraform_compliance.main.mkdtemp', return_value='/tmp/something')
    @patch('terraform_compliance.main.copy_tree', return_value=True)
    @patch('terraform_compliance.main.rmtree', return_value=True)
    @patch('terraform_compliance.main.Repo', return_value=True)
    @patch('terraform_compliance.main.load_tf_files', return_value=True)
    @patch('terraform_compliance.main.call_radish', return_value=True)
    def test_cli_use_remote_dir_for_features(self, *args):
        args = MockedArgHandling()
        args.ssh_key = False
        args.features = 'git:..'
        self.assertTrue(cli(args, MockedArgumentParser('test', 'test description')))

    @patch('terraform_compliance.main.mkdtemp', return_value='/tmp/something')
    @patch('terraform_compliance.main.copy_tree', return_value=True)
    @patch('terraform_compliance.main.rmtree', return_value=True)
    @patch('terraform_compliance.main.Repo', return_value=True)
    @patch('terraform_compliance.main.load_tf_files', return_value=True)
    @patch('terraform_compliance.main.call_radish', return_value=True)
    def test_cli_use_remote_dir_for_terraform_files(self, *args):
        args = MockedArgHandling()
        args.ssh_key = False
        args.tf_dir = 'git:..'
        self.assertTrue(cli(args, MockedArgumentParser('test', 'test description')))



from unittest import TestCase
from terraform_compliance.common.pyhcl_helper import (
    load_tf_files,
    pad_invalid_tf_files,
    pad_tf_file,
    parse_hcl_value
)
from tests.mocks import MockedValidator
from os import remove, path, environ
from mock import patch


class TestPyHCLHelper(TestCase):

    def test_pad_tf_file(self):
        tmpFile = '.terraform_compliance_unit_tests.tf'
        pad_tf_file(tmpFile)
        contents = open(tmpFile, 'r').read()
        remove(tmpFile)
        self.assertEqual(contents, '\n\nvariable {}')

    @patch('terraform_compliance.common.pyhcl_helper.pad_tf_file', return_value=None)
    def test_pad_invalid_tf_files(self, *args):
        self.assertTrue(pad_invalid_tf_files('filename\nUnexpected end of file'))
        self.assertFalse(pad_invalid_tf_files('filename\nAnother message'))

    @patch.object(path, 'isdir', return_value=True)
    def test_load_tf_files_exit_for_dot_terraform(self, *args):
        with self.assertRaises(SystemExit):
            load_tf_files('a_directory')

    @patch.object(path, 'isdir', return_value=False)
    @patch('terraform_compliance.common.pyhcl_helper.Validator', side_effect=MockedValidator)
    def test_load_tf_files_valueerror(self, *args):
        with self.assertRaises(SystemExit):
            load_tf_files('valueerror')

    @patch.object(path, 'isdir', return_value=False)
    @patch('terraform_compliance.common.pyhcl_helper.Validator', side_effect=MockedValidator)
    @patch('terraform_compliance.common.pyhcl_helper.pad_invalid_tf_files', return_value=None)
    def test_load_tf_files_terraformsyntaxexception(self, *args):
        self.assertTrue(load_tf_files('syntaxexception'))
        self.assertEqual(environ['MockedValidator.state'], '1')
        del environ['MockedValidator.state']

    @patch.object(path, 'isdir', return_value=False)
    @patch('terraform_compliance.common.pyhcl_helper.Validator', side_effect=MockedValidator)
    @patch('terraform_compliance.common.pyhcl_helper.pad_invalid_tf_files', return_value=None)
    def test_load_tf_files_success(self, *args):
        self.assertTrue(load_tf_files('passed'))
        self.assertTrue(environ.get('MockedValidator.state', None) is None)


    def test_parse_hcl_value_return_same(self):
        tf_conf = dict(variable=dict(key='value'))
        self.assertEqual(parse_hcl_value('some_string', tf_conf), 'some_string')



from unittest import TestCase
from terraform_compliance.common.helper import (
    flatten_list,
    generate_target_resource,
    expand_variable,
    check_if_cidr,
    is_ip_in_cidr,
    assign_sg_params,
    validate_sg_rule,
    change_value_in_dict
)
from tests.mocks import MockedData
from copy import deepcopy


class TestHelperFunctions(TestCase):

    def test_flatten_single_dimensional_list(self):
        a = ['a', 'b', 'c']
        b = ['a', 'b', 'c']

        self.assertEqual(flatten_list(a), b)

    def test_flatten_multi_dimensional_list(self):
        a = ['a', 'b', ['c']]
        b = ['a', 'b', 'c']

        self.assertEqual(flatten_list(a), b)

    def test_flatten_multi_dimensional_nested_list(self):
        a = ['a', 'b', ['c', ['d', 'e'], 'f'], 'g', 'h', 'i', ['j', 'k']]
        b = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k']

        self.assertEqual(flatten_list(a), b)

    def test_generate_target_resource(self):
        self.assertEqual(['resource', 'target', 'test'], generate_target_resource('target.test'))

    def test_generate_target_resource_has_id_and_name_in_it(self):
        self.assertEqual(['resource', 'target', 'test'], generate_target_resource('target.test.id'))
        self.assertEqual(['resource', 'target', 'test'], generate_target_resource('target.test.name'))

    def test_expand_variable_found(self):
        tf_conf = dict(variable=dict(key='value'))
        self.assertEqual('value', expand_variable(tf_conf, '${var.key}'))

    def test_expand_variable_not_found(self):
        tf_conf = dict(variable=dict(key='value'))
        self.assertEqual('${var.invalid_key}', expand_variable(tf_conf, '${var.invalid_key}'))
        self.assertEqual('${invalid.invalid_key}', expand_variable(tf_conf, '${invalid.invalid_key}'))

    def test_check_if_cidr_success(self):
        self.assertTrue(check_if_cidr('10.0.0.0/8'))
        self.assertTrue(check_if_cidr('10.14.0.0/16'))
        self.assertTrue(check_if_cidr('10.0.0.0/24'))
        self.assertTrue(check_if_cidr('10.0.0.7/32'))

    def test_check_if_cidr_failure(self):
        self.assertFalse(check_if_cidr('256.0.0.0/8'))
        self.assertFalse(check_if_cidr('10.256.0.0/16'))
        self.assertFalse(check_if_cidr('10.0.256.0/24'))
        self.assertFalse(check_if_cidr('10.0.0.256/32'))
        self.assertFalse(check_if_cidr('10.0.0.256/33'))

    def test_is_ip_in_cidr_success(self):
        self.assertTrue(is_ip_in_cidr('10.0.0.0/8', ['0.0.0.0/0']))
        self.assertTrue(is_ip_in_cidr('10.0.0.0/16', ['10.0.0.0/8']))
        self.assertTrue(is_ip_in_cidr('10.0.200.0/24', ['10.0.0.0/16']))
        self.assertTrue(is_ip_in_cidr('10.0.0.1/32', ['10.0.0.0/24']))

    def test_is_ip_in_cidr_failure(self):
        self.assertFalse(is_ip_in_cidr('200.0.0.0/16', ['10.0.0.0/8']))
        self.assertFalse(is_ip_in_cidr('10.200.0.0/24', ['10.0.0.0/16']))
        self.assertFalse(is_ip_in_cidr('10.0.1.1/32', ['10.0.0.0/24']))

    def test_assign_sg_params_one_port_with_two_cidrs(self):
        self.assertEqual(MockedData.sg_params_ssh_with_2_cidrs, assign_sg_params(MockedData.sg_ssh_with_2_cidrs))

    def test_assign_sg_params_one_port_two_cidrs_any_proto(self):
        self.assertEqual(MockedData.sg_params_ssh_with_2_cidrs_any_proto, assign_sg_params(MockedData.sg_ssh_with_2_cidrs_any_proto))

    def test_assign_sg_params_all_ports_with_all_ips(self):
        self.assertEqual(MockedData.sg_params_all_port_all_ip, assign_sg_params(MockedData.sg_all_port_all_ip))

    def test_assign_sg_params_no_data_given_in_rules(self):
        self.assertEqual(MockedData.sg_params_all_port_no_ip, assign_sg_params(MockedData.sg_all_port_no_ip))

    def test_assign_sg_params_from_port_bigger_than_to_port(self):
        with self.assertRaises(AssertionError) as context:
            assign_sg_params(MockedData.sg_invalid)

            self.assertTrue('Invalid configuration from_port can not be bigger than to_port.' in context.exception)

    def test_validate_sg_rule_port_found_in_cidr(self):
        with self.assertRaises(AssertionError) as context:
            validate_sg_rule('tcp', '22', '0.0.0.0/0', MockedData.sg_params_all_port_all_ip)

            self.assertTrue('Found' in context.exception)

    def test_change_value_in_dict_with_str_path(self):
        target_dict = dict(key=dict(another_key='value'))
        change_value_in_dict(target_dict, 'key', dict(added_key='added_value'))
        self.assertEqual(target_dict, dict(key=dict(another_key='value', added_key='added_value')))

    def test_change_value_in_dict_with_dict_path(self):
        target_dict = dict(key=dict(another_key='value'))
        change_value_in_dict(target_dict, ['key'], dict(added_key='added_value'))
        self.assertEqual(target_dict, dict(key=dict(another_key='value', added_key='added_value')))



import sys
import os
from argparse import Action


class ReadableDir(Action):
    def __init__(self, option_strings, dest, nargs=None, const=None, default=None, type=None, choices=None,
                       required=False, help=None, metavar=None):
        # super(ReadableDir, self).__init__(**kwargs) # dest, required, help, option_strings, metavar)
        self.option_strings = option_strings
        self.dest = dest
        self.nargs = nargs
        self.const = const
        self.default = default
        self.type = type
        self.choices = choices
        self.required = required
        self.help = help
        self.metavar = metavar

    def __call__(self, parser, namespace, values, option_string=None):
        prospective_dir = values

        # Check if the given directory is actually a git repo
        if prospective_dir.startswith('git:'):
            print('Using remote git repository: {}'.format(prospective_dir[4:]))
            setattr(namespace, self.dest, prospective_dir[4:])
            return True

        # Check if the given path is a directory really
        if not os.path.isdir(prospective_dir):
            print('ERROR: {} is not a directory.'.format(prospective_dir))
            sys.exit(1)

        # Check if we have access to that directory
        if os.access(prospective_dir, os.R_OK):
            setattr(namespace, self.dest, prospective_dir)
            return True

        print('ERROR: Can not read {}'.format(prospective_dir))
        sys.exit(1)



from unittest import TestCase
from terraform_compliance.steps.steps import (
    i_action_them,
    i_expect_the_result_is_operator_than_number,
    it_condition_contain_something,
    encryption_is_enabled,
    its_value_condition_match_the_search_regex_regex,
    its_value_must_be_set_by_a_variable,
    it_must_not_have_proto_protocol_and_port_port_for_cidr
)
from tests.mocks import MockedStep, MockedWorld, MockedTerraformPropertyList, MockedTerraformResourceList
from mock import patch


class Test_Step_Cases(TestCase):

    def setUp(self):
        self.step = MockedStep()

    def test_i_action_them_count(self):
        step = MockedStep()
        step.context.stash.resource_list = [1,2,3]
        i_action_them(step, 'count')
        self.assertEqual(step.context.stash, 3)

    def test_i_action_them_sum(self):
        step = MockedStep()
        step.context.stash.resource_list = [1,2,3]
        i_action_them(step, 'sum')
        self.assertEqual(step.context.stash, 6)

    def test_i_action_them_undefined(self):
        # with self.assertRaises():
        self.assertIsNone(i_action_them(self.step, 'undefined action'))

    def test_i_action_them_resource_list_as_dict(self):
        step = MockedStep()
        step.context.stash.resource_list = None
        self.assertIsNone(i_action_them(step, 'something that is not important'))

    def test_i_expect_the_result_is_operator_than_number_resource_list_as_dict(self):
        step = MockedStep()
        step.context.stash = 42
        self.assertIsNone(i_expect_the_result_is_operator_than_number(step, 'operator', 'not_important'))

    def test_i_expect_the_result_is_more_than_number_success(self):
        step = MockedStep()
        step.context.stash = 1
        self.assertIsNone(i_expect_the_result_is_operator_than_number(step, 'more', 0))

    def test_i_expect_the_result_is_more_than_number_failure(self):
        step = MockedStep()
        step.context.stash = 1
        with self.assertRaises(AssertionError) as err:
            self.assertIsNone(i_expect_the_result_is_operator_than_number(step, 'more', 1))
        self.assertEqual(str(err.exception), '1 is not more than 1')

    def test_i_expect_the_result_is_more_and_equal_than_number_success(self):
        step = MockedStep()
        step.context.stash = 1
        self.assertIsNone(i_expect_the_result_is_operator_than_number(step, 'more and equal', 0))
        self.assertIsNone(i_expect_the_result_is_operator_than_number(step, 'more and equal', 1))

    def test_i_expect_the_result_is_more_and_equal_than_number_failure(self):
        step = MockedStep()
        step.context.stash = 1
        with self.assertRaises(AssertionError) as err:
            i_expect_the_result_is_operator_than_number(step, 'more and equal', 2)
        self.assertEqual(str(err.exception), '1 is not more and equal than 2')

    def test_i_expect_the_result_is_less_than_number_success(self):
        step = MockedStep()
        step.context.stash = 1
        self.assertIsNone(i_expect_the_result_is_operator_than_number(step, 'less', 2))

    def test_i_expect_the_result_is_less_than_number_failure(self):
        step = MockedStep()
        step.context.stash = 1
        with self.assertRaises(AssertionError) as err:
            i_expect_the_result_is_operator_than_number(step, 'less', 1)
        self.assertEqual(str(err.exception), '1 is not less than 1')

    def test_i_expect_the_result_is_less_and_equal_than_number_success(self):
        step = MockedStep()
        step.context.stash = 1
        self.assertIsNone(i_expect_the_result_is_operator_than_number(step, 'less and equal', 1))
        self.assertIsNone(i_expect_the_result_is_operator_than_number(step, 'less and equal', 2))

    def test_i_expect_the_result_is_less_and_equal_than_number_failure(self):
        step = MockedStep()
        step.context.stash = 1
        with self.assertRaises(AssertionError) as err:
            i_expect_the_result_is_operator_than_number(step, 'less and equal', 0)
        self.assertEqual(str(err.exception), '1 is not less and equal than 0')

    def test_i_expect_the_result_is_invalid_operator_than_number_failure(self):
        step = MockedStep()
        step.context.stash = 1
        self.assertIsNone(i_expect_the_result_is_operator_than_number(step, 'invalid_operator', 0))

    def test_it_condition_contain_something_resource_list(self):
        step = MockedStep()
        step.context.stash.resource_list = None
        self.assertIsNone(it_condition_contain_something(step, 'should', 'not_important'))

    @patch('terraform_compliance.steps.steps.world', side_effect=MockedWorld())
    def test_it_must_contain_something_property_can_not_be_found(self, *args):
        step = MockedStep()
        step.context.stash = MockedTerraformPropertyList()
        step.sentence = 'Then it must contain'
        with self.assertRaises(AssertionError) as err:
            it_condition_contain_something(step, 'non_existent_property_value', MockedTerraformPropertyList)
        self.assertEqual(str(err.exception), 'non_existent_property_value property in test_name can not be found in ' 
                                             'test_resource_name (test_resource_type). It is set to test_value instead')

    def test_it_condition_must_something_property_can_not_be_found(self):
        step = MockedStep()
        step.context.stash = MockedTerraformResourceList()
        step.sentence = 'Then it must ..'
        with self.assertRaises(Exception) as err:
            it_condition_contain_something(step=step, something=None, resourcelist=MockedTerraformResourceList)
        self.assertEqual(str(err.exception), 'should_have_properties hit')

        step.sentence = 'When it contains'
        it_condition_contain_something(step=step, something=None, resourcelist=MockedTerraformResourceList)
        self.assertEqual(step.state, 'skipped')

    def test_it_condition_must_something_property_is_found(self):
        step = MockedStep()
        step.context.stash = MockedTerraformResourceList()
        step.sentence = 'Then it must ..'
        it_condition_contain_something(step=step, something='something', resourcelist=MockedTerraformResourceList)
        self.assertEqual(step.context.stash[0].__class__, MockedTerraformPropertyList)

    def test_it_condition_must_something_property_stash_is_dict_found(self):
        step = MockedStep()
        step.context.stash = {'something': 'something else'}
        self.assertIsNone(it_condition_contain_something(step=step, something='something', resourcelist=MockedTerraformResourceList))

    def test_it_condition_should_something_property_stash_is_dict_found(self):
        step = MockedStep()
        step.context.stash = {}
        step.sentence = 'Then it must contain'
        with self.assertRaises(AssertionError) as err:
            it_condition_contain_something(step=step, something='something', resourcelist=MockedTerraformResourceList)
        self.assertEqual(str(err.exception), 'something does not exist.')

        step.sentence = 'When it contains'
        step.context.stash = {}
        it_condition_contain_something(step=step, something='something', resourcelist=MockedTerraformResourceList)
        self.assertEqual(step.state, 'skipped')

    def test_encryption_is_enabled_resource_list(self):
        step = MockedStep()
        step.context.stash = MockedTerraformResourceList()
        self.assertIsNone(encryption_is_enabled(step))

    def test_its_value_condition_match_the_search_regex_regex_resource_list(self):
        step = MockedStep()
        step.context.stash = MockedTerraformPropertyList()
        self.assertIsNone(its_value_condition_match_the_search_regex_regex(step, 'condition', 'some_regex'))

    def test_its_value_must_match_the_search_regex_regex_string_unicode_success(self):
        step = MockedStep()
        step.context.stash = 'some string'
        self.assertIsNone(its_value_condition_match_the_search_regex_regex(step, 'must', '^[sometring\s]+$'))

    def test_its_value_must_match_the_search_regex_regex_string_unicode_failure(self):
        step = MockedStep()
        step.context.stash = 'some string'
        step.context.name = 'test name'
        step.context.type = 'test type'
        with self.assertRaises(AssertionError) as err:
            its_value_condition_match_the_search_regex_regex(step, 'must', 'non_match_regex')
        self.assertEqual(str(err.exception), '{} {} tests failed on {} regex: {}'.format(step.context.name,
                                                                                         step.context.type,
                                                                                         'non_match_regex',
                                                                                         step.context.stash))

    def test_its_value_must_match_not_the_search_regex_regex_string_unicode_success(self):
        step = MockedStep()
        step.context.stash = 'some string'
        self.assertIsNone(its_value_condition_match_the_search_regex_regex(step, 'must not', 'non_match_regex'))

    def test_its_value_must_not_match_the_search_regex_regex_string_unicode_failure(self):
        step = MockedStep()
        step.context.stash = 'some string'
        step.context.name = 'test name'
        step.context.type = 'test type'
        with self.assertRaises(AssertionError) as err:
            its_value_condition_match_the_search_regex_regex(step, 'must not', '^[sometring\s]+$')
        self.assertEqual(str(err.exception), '{} {} tests failed on {} regex: {}'.format(step.context.name,
                                                                                         step.context.type,
                                                                                         '^[sometring\s]+$',
                                                                                         step.context.stash))

    def test_its_value_must_match_the_search_regex_regex_success(self):
        step = MockedStep()
        step.context.stash = MockedTerraformPropertyList()
        self.assertIsNone(its_value_condition_match_the_search_regex_regex(step, 'must', '^[tesvalu_\s]+$'))

    def test_its_value_must_match_the_search_regex_regex_failure(self):
        step = MockedStep()
        step.context.stash = MockedTerraformPropertyList()
        with self.assertRaises(AssertionError):
            its_value_condition_match_the_search_regex_regex(step, 'must', 'non_match_regex')

    def test_its_value_must_not_match_the_search_regex_regex_success(self):
        step = MockedStep()
        step.context.stash = MockedTerraformPropertyList()
        self.assertIsNone(its_value_condition_match_the_search_regex_regex(step, 'must not', '^[tesvalu_\s]+$'))

    def test_its_value_must_not_match_the_search_regex_regex_failure(self):
        step = MockedStep()
        step.context.stash = MockedTerraformPropertyList()
        with self.assertRaises(AssertionError):
            its_value_condition_match_the_search_regex_regex(step, 'must not', 'non_match_regex')

    def test_its_value_must_be_set_by_a_variable_resource_list(self):
        step = MockedStep()
        step.context.stash = MockedTerraformResourceList()
        step.context.search_value = 'something'
        self.assertIsNone(its_value_must_be_set_by_a_variable(step))

    @patch.object(MockedTerraformResourceList, 'property', return_value=MockedTerraformResourceList())
    def test_its_value_must_be_set_by_a_variable(self, *args):
        step = MockedStep()
        step.context.stash = MockedTerraformResourceList()
        step.context.search_value = MockedTerraformResourceList()
        self.assertIsNone(its_value_must_be_set_by_a_variable(step))



from sys import exc_info, exit
from os.path import isdir
from terraform_compliance import Validator
from terraform_validate.terraform_validate import TerraformSyntaxException
from terraform_compliance.common.exceptions import TerraformComplianceInvalidConfig
from terraform_compliance.common.helper import expand_variable
from shutil import rmtree
from hcl import loads


def load_tf_files(tf_directory):
    result = False
    print('Reading terraform files.')

    if isdir('{}/.terraform'.format(tf_directory)):
        rmtree('{}/.terraform'.format(tf_directory))

    while result is False:
        try:
            Validator(tf_directory)
            print('All HCL files look good.')
            return True

        except ValueError:
            print('Unable to validate Terraform Files.')
            print('ERROR: {}'.format(exc_info()[1]))
            exit(1)

        except TerraformSyntaxException:
            result = pad_invalid_tf_files(exc_info()[1]) is True

        except TerraformComplianceInvalidConfig:
            pass

    return result


def pad_invalid_tf_files(exception_message):
    exception_message = str(exception_message).split('\n')
    if 'Unexpected end of file' in exception_message[1]:
        filename = exception_message[0].split(' ')[-1:][0]
        print('Invalid HCL file: {}. Fixing it.'.format(filename))
        pad_tf_file(filename)
        return True
    elif 'Invalid terraform configuration in ' in exception_message[0]:
        raise TerraformComplianceInvalidConfig('ERROR: Invalid terraform configuration {}'.format(exception_message[1]))

    return False


def pad_tf_file(file):
    with open(file, 'a') as f:
        f.write('\n\nvariable {}')


def parse_hcl_value(hcl_string, tf_config):
    if str(hcl_string).startswith(('${', '"${')):
        hcl = "key = \"{}\"".format(str(hcl_string).replace("'", "\"").lower())

        try:
            hcl = loads(hcl)
            if "." in hcl['key']:
                hcl['key'] = expand_variable(tf_conf=tf_config, value=hcl['key'])
            return eval(hcl['key'].replace('${', '{'))
        except ValueError:
            return hcl_string

    if type(hcl_string) is dict:
        hcl_string = {k.lower(): v for k, v in hcl_string.items()}

    return hcl_string



import re
from netaddr import IPNetwork


hcl_conditions = ["==", "!=", ">", "<", ">=", "<=", "&&", "||", "!"]


# A helper function that will be used to flatten a multi-dimensional multi-nested list
def flatten_list(input):
    new_list = []
    for i in input:
        if type(i) is list:
            i = flatten_list(i)
            for j in i:
                new_list.extend(j)
        else:
            new_list.extend(i)
    return new_list


def generate_target_resource( target_string ):
    target = target_string.strip()
    target = 'resource.{}'.format(target).split('.')

    if target[-1] in ['id', 'name']:
        target.pop(-1)

    return target


def expand_variable(tf_conf, value):
    regex = r'\${var.(.*)\}'
    matches = re.match(regex, value)

    if matches is None:
        if 'module' in value:
            return 'None'

        return value

    if matches.group(1) not in tf_conf['variable']:
        return value

    return tf_conf['variable'][matches.group(1)]


def check_if_cidr( value ):
    regex = r'(1[0-9][0-9]|2[0-4][0-9]|25[0-5]|[0-9][0-9]|[0-9])\.(1[0-9][0-9]|2[0-4][0-9]|25[0-5]|[0-9][0-9]|[0-9])\.(1[0-9][0-9]|2[0-4][0-9]|25[0-5]|[0-9][0-9]|[0-9])\.(1[0-9][0-9]|2[0-4][0-9]|25[0-5]|[0-9][0-9]|[0-9])\/(3[0-2]|2[0-9]|1[0-9]|[0-9])'
    matches = re.match(regex, value)

    if matches is not None:
        return True

    return False


def is_ip_in_cidr(ip_cidr, cidr):
    for ip_network in cidr:
        if check_if_cidr(ip_cidr) and check_if_cidr(ip_network):
            if IPNetwork(ip_cidr) in IPNetwork(ip_network):
                return True

    return False


# A helper function that compares port related data with given dictionary
def check_sg_rules(tf_conf, security_group, proto, port, cidr):

    if 'cidr_blocks' in security_group:
        if type(security_group['cidr_blocks']) is list:
            for i in range(0,len(security_group['cidr_blocks'])):
                if not check_if_cidr(security_group['cidr_blocks'][i]):
                    security_group['cidr_blocks'][i] = expand_variable(tf_conf,
                                                                       security_group['cidr_blocks'][i]
                                                                       ).get('default',
                                                                             security_group['cidr_blocks'][i])
        else:
            if not check_if_cidr(security_group['cidr_blocks']):
                security_group['cidr_blocks'] = expand_variable(tf_conf,
                                                                security_group['cidr_blocks']
                                                                ).get('default',
                                                                      security_group['cidr_blocks'])


    validate_sg_rule(proto=proto, port=port, cidr=cidr, params=assign_sg_params(security_group))


def assign_sg_params(rule):
    from_port = int(rule.get('from_port', 0))
    to_port = int(rule.get('to_port', 0))

    protocol = [proto for proto in [rule.get('protocol', '-1')]]

    # TODO: Make IANA Protocol numbers matching here.
    # http://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml
    if protocol[0] == '-1' or type(protocol[0]) is int:
        protocol = ['tcp', 'udp']

    protocol[0] = protocol[0].lower()

    cidr_blocks = rule.get('cidr_blocks', [])

    if type(cidr_blocks) is not list:
        cidr_blocks = [cidr_blocks]

    if to_port == 0 and from_port == 0:
        to_port = 65535

    if from_port > to_port:
        raise AssertionError('Invalid configuration from_port can not be bigger than to_port. {} > {} {} in {}'.format(from_port,
                                                                                                                 to_port,
                                                                                                                 protocol,
                                                                                                                 cidr_blocks))

    return dict(protocol=protocol, from_port=from_port, to_port=to_port, cidr_blocks=cidr_blocks)


def validate_sg_rule(proto, port, cidr, params):
    port = int(port)
    if port >= params['from_port'] and port <= params['to_port'] and proto in params['protocol'] and is_ip_in_cidr(cidr, params['cidr_blocks']):
        raise AssertionError('Found {}/{} in {}/{}-{} for {}'.format(proto, port, params['protocol'], params['from_port'], params['to_port'], params['cidr_blocks']))


def change_value_in_dict(target_dictionary, path_to_change, value_to_change):
    if type(path_to_change) is str:
        path_to_change = path_to_change.split('.')

    if type(path_to_change) is not list:
        return False

    for x in xrange(0,len(path_to_change)):
        for condition in hcl_conditions:
            if condition in path_to_change[x]:
                return False

    path_to_adjust = '["{}"]'.format('"]["'.join(path_to_change))

    try:
        target = eval('target_dictionary{}'.format(path_to_adjust))

        for key, value in value_to_change.items():
            if 'type' in value:
                type_key = value['type']
                source = value
                source['referenced_name'] = key

                if type_key not in target:
                    target[type_key] = list()
                elif type_key in target and type(target[type_key]) is not list:
                        target[type_key] = [target[type_key]]

                target[type_key].append(source)

        target.update(value_to_change)

        try:
            exec('target_dictionary{}.update({})'.format(path_to_adjust, target))
        except:
            # Yes I know, this is against PEP8.
            pass

    except KeyError:
        pass


def strip_conditions(string):
    for condition in hcl_conditions:
        string = string.replace(condition, "")


    string = string.split(" ")

    return string[0]





untaggable_resources = [
    "aws_route_table",
    "aws_route",
    "aws_main_route_table_association",
    "aws_elastic_beanstalk",
    "aws_elasticsearch_domain_policy",
    "aws_security_group_rule",
    "aws_eip",
    "aws_nat_gateway",
    "aws_egress_only_internet_gateway",
    "aws_key_pair",
    "aws_lambda",
    "aws_iam",
    "aws_s3_bucket_notification",
    "aws_api_gateway",
    "aws_cloudfront_origin_access_identity",
    "aws_cloudwatch",
    "aws_server_certificate",
    "aws_route53_record",
    "aws_directory_service_directory",
    "azurerm_resource_group",
    "aws_efs_mount_target",
    "aws_ecs_cluster",
    "aws_launch_configuration",
    "aws_kms_alias",
    "aws_kinesis_firehose_delivery_stream",
    "aws_vpc_dhcp_options_association",
    "null_resource"
]

encryption_property = {
    "aws_db_instance": "storage_encrypted",
    "ebs_block_device": "encrypted",
    "aws_ebs_volume": "encrypted",
    "azurerm_storage_account": "enable_blob_encryption",
    "azurerm_sql_database": "encryption"
}

resource_name = {
    "AWS Auto-Scaling Group": "aws_autoscaling_group",
    "AWS EBS volume": "aws_ebs_volume",
    "AWS EC2 instance": "aws_instance",
    "AWS EFS file system": "aws_efs_file_system",
    "AWS ELB resource": "aws_elb",
    "AWS Internet Gateway": "aws_internet_gateway",    
    "AWS Launch Configuration": "aws_launch_configuration",
    "AWS RDS instance": "aws_db_instance",
    "AWS Route Table": "aws_route_table",
    "AWS S3 Bucket": "aws_s3_bucket",
    "AWS Security Group": "aws_security_group",
    "AWS VPC": "aws_vpc",
    "Azure SQL Database": "azurerm_sql_database",
    "Azure Storage Account": "azurerm_storage_account",
    "AWS Subnet": "aws_subnet",
    "resource that supports tags": "(?!{0}).*".format("|".join(untaggable_resources))
}

regex = {
    "Name": "^\${var.platform}_\${var.environment}_.*"
}



# -*- coding: utf-8 -*-

from radish import step, world, custom_type, given, when, then
from terraform_compliance.steps import resource_name, encryption_property
from terraform_compliance.common.helper import check_sg_rules
from terraform_compliance.common.pyhcl_helper import parse_hcl_value
from terraform_compliance.extensions.terraform_validate import normalise_tag_values
from terraform_validate.terraform_validate import TerraformPropertyList, TerraformResourceList
from terraform_compliance.extensions.ext_radish_bdd import skip_step, step_condition, write_stdout
import re

# world.config.debug_steps = True

# New Arguments
@custom_type("ANY", r"[\.\/_\-A-Za-z0-9\s]+")
def custom_type_any(text):
    return text

@custom_type("SECTION", r"[a-z]+")
def custom_type_section(text):
    if text in ['resource', 'provider', 'data', 'module', 'output', 'terraform', 'variable']:
        return text

@given(u'I have {name:ANY} {type:SECTION} configured')
def i_have_name_section_configured(step, name, type, radish_world=None):
    if radish_world is None:
        radish_world = world

    step.context.type = type
    step.context.name = name

    if type == "resource":
        if (name in resource_name.keys()):
            name = resource_name[name]

        found_resource = radish_world.config.terraform.resources(name)

        if hasattr(found_resource, 'resource_list') and found_resource.resource_list:
            step.context.resource_type = name
            step.context.defined_resource = name
            step.context.stash = radish_world.config.terraform.resources(name)
        else:
            skip_step(step, name)
    else:
        if type in radish_world.config.terraform.terraform_config:
            if name in radish_world.config.terraform.terraform_config[type]:
                step.context.stash = radish_world.config.terraform.terraform_config[type][name]
            else:
                step.context.stash = radish_world.config.terraform.terraform_config[type]

        else:
            skip_step(step, type)

@given(u'I have {resource:ANY} defined')
def i_have_resource_defined(step, resource, radish_world=None):
    if radish_world is None:
        radish_world = world

    if (resource in resource_name.keys()):
        resource = resource_name[resource]

    found_resource = radish_world.config.terraform.resources(resource)

    if found_resource.resource_list:
        step.context.resource_type = resource
        step.context.defined_resource = resource
        step.context.stash = radish_world.config.terraform.resources(resource)
    else:
        skip_step(step, '{} resource'.format(resource))

@when(u'I {action_type:ANY} them')
def i_action_them(step, action_type):
    if action_type == "count":
        step.context.stash = len(step.context.stash.resource_list)
    elif action_type == "sum":
        step.context.stash = sum(step.context.stash.resource_list)
    else:
        AssertionError("Invalid action_type in the scenario: {}".format(action_type))


@then(u'I expect the result is {operator:ANY} than {number:d}')
def i_expect_the_result_is_operator_than_number(step, operator, number):
    value = int(step.context.stash)

    if operator == "more":
        assert value > number, "{} is not more than {}".format(value, number)
    elif operator == "more and equal":
        assert value >= number, "{} is not more and equal than {}".format(value, number)
    elif operator == "less":
        assert value < number, "{} is not less than {}".format(value, number)
    elif operator == "less and equal":
        assert value <= number, "{} is not less and equal than {}".format(value, number)
    else:
        AssertionError('Invalid operator: {}'.format(operator))

@when(u'it contain {something:ANY}')
@when(u'it contains {something:ANY}')
@then(u'it must contain {something:ANY}')
def it_condition_contain_something(step, something,
                                   propertylist=TerraformPropertyList, resourcelist=TerraformResourceList):

    if something in resource_name.keys():
        something = resource_name[something]

    step_can_skip = step_condition(step) in ["given", "when"]

    if step.context.stash.__class__ is propertylist:
        for property in step.context.stash.properties:
            value = parse_hcl_value(property.property_value, world.config.terraform.terraform_config)

            if value is not None:
                assert (value == something or something.lower() in value), \
                    '{} property in {} can not be found in {} ({}). It is set to {} instead'.format(something,
                                                                                                    property.property_name,
                                                                                                    property.resource_name,
                                                                                                    property.resource_type,
                                                                                                    value)
            else:
                write_stdout(level='WARNING',
                             message='Can not get value of {} in {}/{}. '
                                     'Might be set by an unknown source (module, etc. )\n'
                                     'Value : {}'.format(something,
                                                             property.property_name,
                                                             property.resource_type,
                                                             property.property_value))
                step.state = 'skipped'

    elif step.context.stash.__class__ is resourcelist:
        if step_can_skip is False:
            step.context.stash.should_have_properties(something)
            step.context.stash = step.context.stash.find_property(something)
            assert step.context.stash.properties, \
                'No defined property/value found for {}.'.format(something)
            step.context.stash = step.context.stash.properties
        else:
            try:
                step.context.stash.should_have_properties(something)
                number_of_resources = len(step.context.stash.resource_list)
                step.context.stash = step.context.stash.find_property(something)
                if step.context.stash:
                    if number_of_resources > len(step.context.stash.properties):
                        write_stdout(level='INFO',
                                     message='Some of the resources does not have {} property defined within.\n'
                                             'Removed {} resource (out of {}) from the test scope.\n\n'.format(something,
                                                                    (number_of_resources-len(step.context.stash.properties)),
                                                                                                               number_of_resources,
                                                                    ))
            except Exception as e:
                number_of_resources = len(step.context.stash.resource_list)
                step.context.stash = step.context.stash.find_property(something)
                if step.context.stash:
                    write_stdout(level='INFO',
                                 message='Some of the resources does not have {} property defined within.\n' 
                                         'Removed {} resource (out of {}) from the test scope.\n\n'
                                         'Due to : \n{}'.format(something,
                                                    (number_of_resources-len(step.context.stash.properties)),
                                                                number_of_resources,
                                                    str(e)))
                else:
                    skip_step(step,
                              resource=something,
                              message='Can not find {resource} property in any resource.')

    elif step.context.stash.__class__ is dict:
        if something in step.context.stash:
            step.context.stash = step.context.stash[something]
        else:
            if step_can_skip:
                skip_step(step,
                          resource=something,
                          message='Can not find {resource} resource in terraform files.')
            else:
                assert False, '{} does not exist.'.format(something)

@then(u'encryption is enabled')
@then(u'encryption must be enabled')
def encryption_is_enabled(step):
    prop = encryption_property[step.context.resource_type]
    step.context.stash.property(prop).should_equal(True)


@then(u'its value {condition} match the "{search_regex}" regex')
def its_value_condition_match_the_search_regex_regex(step, condition, search_regex):
    regex = r'{}'.format(search_regex)

    if step.context.stash.__class__ in (str, unicode):
        matches = re.match(regex, step.context.stash)

        if condition == 'must':
            assert matches is not None, \
                '{} {} tests failed on {} regex: {}'.format(step.context.name,
                                                            step.context.type,
                                                            regex,
                                                            step.context.stash)
        elif condition == "must not":
            assert matches is None, \
                '{} {} tests failed on {} regex: {}'.format(step.context.name,
                                                            step.context.type,
                                                            regex,
                                                            step.context.stash)
    else:
        normalise_tag_values(step.context.stash)


        for property in step.context.stash.properties:
            if type(property.property_value) in [str, unicode]:
                property.property_value = [property.property_value]
            elif type(property.property_value) is dict:
                property.property_value = property.property_value.values()

            for value in property.property_value:
                matches = re.match(regex, value)

                if condition == 'must':
                    assert matches is not None, \
                        '{} property in {} does not match with {} regex. It is set to {} instead.'.format(property.property_name,
                                                                                                      property.resource_name,
                                                                                                      search_regex,
                                                                                                      value)
                elif condition == 'must not':
                    assert matches is not None, \
                        '{} property in {} does not match with {} regex. It is set to {} instead.'.format(property.property_name,
                                                                                                          property.resource_name,
                                                                                                          search_regex,
                                                                                                          value)

@step(u'its value must be set by a variable')
def its_value_must_be_set_by_a_variable(step):
    step.context.stash.property(step.context.search_value).should_match_regex(r'\${var.(.*)}')


@then(u'it must not have {proto} protocol and port {port:d} for {cidr:ANY}')
def it_must_not_have_proto_protocol_and_port_port_for_cidr(step, proto, port, cidr):
    proto = str(proto)
    port = int(port)
    cidr = str(cidr)

    for item in step.context.stash.properties:
        if type(item.property_value) is list:
            for security_group in item.property_value:
                check_sg_rules(world.config.terraform.terraform_config, security_group, proto, port, cidr)
        else:
            check_sg_rules(world.config.terraform.terraform_config, item.property_value, proto, port, cidr)


import re
import sys
from terraform_compliance.common.helper import generate_target_resource
from terraform_compliance.common.helper import change_value_in_dict

if sys.version_info[0] > 2:
    unicode = str


def enable_resource_mounting(tf_conf, processing_resource=None, resource=None):
    if not resource:
        resource = 'resource'
    else:
        resource = resource.strip()

    if not processing_resource:
        if resource not in tf_conf:
            return

        processing_resource = tf_conf[resource]


    source = resource.strip()
    source = (source[:-3] if source.endswith('.id') else source)

    for sub_resource, sub_value in processing_resource.items():

        if type(sub_value) is dict:
            enable_resource_mounting(tf_conf, sub_value, sub_resource)
        else:
            regex = r'\${(.*)\}'
            if type(sub_value) is str or type(sub_value) is unicode:
                matches = re.match(regex, sub_value)

                if matches is not None and not matches.group(1).startswith(('var', 'data', 'module')) and '(' not in matches.group(1):
                    target = generate_target_resource(matches.group(1))
                    change_value_in_dict(tf_conf, target, {source: processing_resource})

            elif type(sub_value) is list:
                for value in sub_value:
                    if type(value) is str or type(value) is unicode:
                        matches = re.match(regex, value)
                        if matches is not None and not matches.group(1).startswith(('var', 'data', 'module')) and '(' not in matches.group(1):
                                target = generate_target_resource(matches.group(1))
                                change_value_in_dict(tf_conf, target, {source: processing_resource})


def normalise_tag_values(property_list):
    for property_object in property_list.properties:
        if type(property_object.property_value) is not dict():
            property_object.property_value = dict(normalised=property_object.property_value)


from unittest import TestCase
from terraform_compliance.extensions.terraform_validate import enable_resource_mounting
from terraform_compliance.extensions.terraform_validate import change_value_in_dict
from tests.mocks import MockedData
from copy import deepcopy


class TestTerraformValidate(TestCase):

    def test_change_value_in_dict_path_to_value_is_str(self):
        target_dictionary = deepcopy(MockedData.target_dictionary)
        change_value_in_dict(target_dictionary,
                             MockedData.path_to_change_as_string,
                             MockedData.target_value)
        self.assertEqual(target_dictionary, MockedData.expected_target_dictionary)

    def test_change_value_in_dict_path_to_value_is_list(self):
        target_dictionary = deepcopy(MockedData.target_dictionary)
        change_value_in_dict(target_dictionary,
                             MockedData.path_to_change_as_list,
                             MockedData.target_value)
        self.assertEqual(target_dictionary, MockedData.expected_target_dictionary)

    def test_enable_resource_mounting_success(self):
        mocked_tf = deepcopy(MockedData.mocked_tf_conf_for_mounting)
        enable_resource_mounting(mocked_tf)

        self.assertTrue('test_ingress' in mocked_tf['resource']['aws_security_group']['test_instance'])
        self.assertTrue('test_egress' in mocked_tf['resource']['aws_security_group']['test_instance'])
        self.assertTrue('default' in MockedData.expected_tf_conf_for_mounting['resource']['aws_security_group'])

    def test_enable_resource_mounting_success_while_there_is_no_mounting(self):
        mocked_tf = deepcopy(MockedData.mocked_tf_conf_with_no_mounting)
        enable_resource_mounting(mocked_tf)

        self.assertFalse('test_ingress' in mocked_tf['resource']['aws_security_group']['test_instance'])
        self.assertFalse('test_egress' in mocked_tf['resource']['aws_security_group']['test_instance'])
        self.assertTrue('default' in MockedData.mocked_tf_conf_with_no_mounting['resource']['aws_security_group'])

    def test_check_if_ingress_egress_is_created(self):
        mocked_tf = deepcopy(MockedData.mocked_tf_conf_for_mounting)
        enable_resource_mounting(mocked_tf)

        self.assertTrue('ingress' in mocked_tf['resource']['aws_security_group']['test_instance'])
        self.assertTrue(type(mocked_tf['resource']['aws_security_group']['test_instance']['ingress']) is list)
        self.assertTrue('egress' in mocked_tf['resource']['aws_security_group']['test_instance'])
        self.assertTrue(type(mocked_tf['resource']['aws_security_group']['test_instance']['egress']) is list)
        self.assertTrue('default' in MockedData.expected_tf_conf_for_mounting['resource']['aws_security_group'])



from unittest import TestCase
from terraform_compliance.steps.steps import i_have_name_section_configured, i_have_resource_defined
from tests.mocks import MockedStep, MockedWorld, MockedWorldConfigTerraform


class Test_Given_Step_Cases(TestCase):

    def setUp(self):
        self.step = MockedStep()
        self.radish_world = MockedWorld()

    def test_i_have_name_section_configured(self):
        i_have_name_section_configured(self.step, 'resource_type', 'resource', self.radish_world)
        self.assertEqual(self.step.context.stash.resource_list[0], MockedWorldConfigTerraform().terraform_config['resource']['resource_type'])

        i_have_name_section_configured(self.step, 'aws', 'provider', self.radish_world)
        self.assertEqual(self.step.context.stash, MockedWorldConfigTerraform().terraform_config['provider']['aws'])

        i_have_name_section_configured(self.step, 'non_existent', 'something_else', self.radish_world)
        self.assertEqual(self.step.context.stash, MockedWorldConfigTerraform().terraform_config['something_else'])

        i_have_name_section_configured(self.step, 'AWS S3 Bucket', 'resource', self.radish_world)
        self.assertEqual(self.step.context.stash.resource_list[0], MockedWorldConfigTerraform().terraform_config['resource']['aws_s3_bucket'])

    def test_i_have_resource_defined(self):
        i_have_resource_defined(self.step, 'resource_type', self.radish_world)
        self.assertEqual(self.step.context.stash.resource_list[0], MockedWorldConfigTerraform().terraform_config['resource']['resource_type'])

        i_have_resource_defined(self.step, 'AWS S3 Bucket', self.radish_world)
        self.assertEqual(self.step.context.stash.resource_list[0], MockedWorldConfigTerraform().terraform_config['resource']['aws_s3_bucket'])




import os
from argparse import ArgumentParser
from sys import exc_info, exit, executable

try:
    from radish.main import main as call_radish
except ImportError as e:
    import subprocess


    def pip(action, package, params=None):
        print '{}ing {}..'.format(action, package)

        if action == 'uninstall':
            cmds = [executable, "-m", "pip", action, '--yes', package]
        else:
            cmds = [executable, "-m", "pip", action, package]

        subprocess.call(cmds)

    print "Fixing the problem on radish and radish-bdd"
    pip('uninstall', 'radish-bdd')
    pip('uninstall', 'radish')
    pip('install', 'radish==0.1.10')
    pip('install', 'radish-bdd==0.8.6')

    print "~"*40
    print " Please run terraform-compliance again."
    print "~"*40
    exit(1)

from tempfile import mkdtemp
from git import Repo
from terraform_compliance.common.pyhcl_helper import load_tf_files
from distutils.dir_util import copy_tree
from shutil import rmtree
from terraform_compliance.common.readable_dir import ReadableDir
from terraform_compliance.common.exceptions import TerraformComplianceInvalidConfig


__app_name__ = "terraform-compliance"
__version__ = "0.5.1"



class ArgHandling(object):
    pass

def cli(arghandling=ArgHandling(), argparser=ArgumentParser(prog=__app_name__,
                                                            description="BDD Test Framework for Hashicorp terraform")):
    args = arghandling
    parser = argparser
    parser.add_argument("--features", "-f", dest="features", metavar='feature_directory', action=ReadableDir,
                        help="Directory consists of BDD features", required=True)
    parser.add_argument("--tfdir", "-t", dest="tf_dir", metavar='terraform_directory', action=ReadableDir,
                        help="Directory (or git repository with 'git:' prefix) consists of Terraform Files",
                        required=True)
    parser.add_argument("--identity", "-i", dest="ssh_key", metavar='ssh_private_key', type=str, nargs='?',
                        help="SSH Private key that will be use on git authentication.", required=False)
    parser.add_argument("--version", "-v", action="version", version=__version__)

    _, radish_arguments = parser.parse_known_args(namespace=args)

    print('{} v{} initiated'.format(__app_name__, __version__))

    steps_directory = os.path.join(os.path.split(os.path.abspath(__file__))[0], 'steps')
    print('Steps    : {}'.format(steps_directory))

    # SSH Key is given for git authentication
    ssh_cmd = {}
    if args.ssh_key:
        ssh_cmd = {"GIT_SSH_COMMAND": "ssh -l {} -i {}".format('git', args.ssh_key)}

    # A remote repository used here
    if args.features.startswith(('http', 'https', 'ssh')):
        features_git_repo = args.features
        args.features = mkdtemp()

        Repo.clone_from(url=features_git_repo, to_path=args.features, env=ssh_cmd)
    features_directory = os.path.join(os.path.abspath(args.features))
    print('Features : {}{}'.format(features_directory, (' ({})'.format(features_git_repo) if 'features_git_repo' in locals() else '')))

    tf_tmp_dir = mkdtemp()

    # A remote repository is used here.
    if args.tf_dir.startswith(('http', 'https', 'ssh')):
        tf_git_repo = args.tf_dir
        Repo.clone_from(url=tf_git_repo, to_path=tf_tmp_dir, env=ssh_cmd)

    # A local directory is used here
    else:
        # Copy the given local directory to another place, since we may change some tf files for compatibility.
        copy_tree(args.tf_dir, tf_tmp_dir)

    tf_directory = os.path.join(os.path.abspath(tf_tmp_dir))
    print('TF Files : {} ({})'.format(tf_directory, args.tf_dir))

    commands = ['radish',
                '--write-steps-once',
                features_directory,
                '--basedir', steps_directory,
                '--user-data=tf_dir={}'.format(tf_directory)]
    commands.extend(radish_arguments)

    try:
        load_tf_files(tf_directory)
    except TerraformComplianceInvalidConfig:
        print exc_info()[1]
        exit(1)

    print('Running tests.')
    result = call_radish(args=commands[1:])

    # Delete temporary directory we created
    print('Cleaning up.')
    rmtree(tf_directory)

    return result

if __name__ == '__main__':
    cli()



from radish import before, world
from terraform_compliance import Validator
from terraform_compliance.extensions.terraform_validate import enable_resource_mounting


@before.each_feature
def load_terraform_data(feature):
    tf_conf = Validator(world.config.user_data['tf_dir'])
    tf_conf.enable_variable_expansion()

    # Custom extensions
    enable_resource_mounting(tf_conf.terraform_config)

    world.config.terraform = tf_conf
